import csv
from io import StringIO
from werkzeug.utils import secure_filename
from flask import Flask, jsonify, render_template, request, redirect, url_for, flash
from flask_sqlalchemy import SQLAlchemy 
from datetime import datetime, timedelta
from datetime import date
from sqlalchemy import or_
from flask_mail import Mail, Message
from flask_apscheduler import APScheduler
from flask import session
from sqlalchemy.exc import IntegrityError



USER_CREDENTIALS = {'username': 'user', 'password': 'user'}

# Initialize the Flask application
app = Flask(__name__)

# Configure the SQLAlchemy part of the app instance
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///iptv_clients.db'
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.config['SECRET_KEY'] = '1hsqdqsddqscqs3423fsd'
app.config['MAX_CONTENT_LENGTH'] = 16 * 1024 * 1024  # 16MB


# Email configuration - Replace with your actual details
app.config['MAIL_SERVER'] = 'versanstr.store'
app.config['MAIL_PORT'] = 465
app.config['MAIL_USE_SSL'] = True
app.config['MAIL_USERNAME'] = 'admin@versanstr.store'
app.config['MAIL_PASSWORD'] = 'Mohamed@2000'


# Initialize the database with the Flask app
db = SQLAlchemy(app)
mail = Mail(app)

# APScheduler configuration
scheduler = APScheduler()
scheduler.init_app(app)

# Define the Client model
class Client(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    client_number = db.Column(db.String(50), unique=True, nullable=False)
    phone = db.Column(db.String(50), nullable=False)
    email = db.Column(db.String(100), nullable=False)
    iptv_username = db.Column(db.String(50), nullable=False)
    panel_name = db.Column(db.String(50), nullable=False)
    expiry_date = db.Column(db.Date, nullable=False)

class PanelName(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(50), unique=True, nullable=False)

class EmailLog(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    recipient = db.Column(db.String(100), nullable=False)
    subject = db.Column(db.String(200), nullable=False)
    timestamp = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    status = db.Column(db.String(50), nullable=False)

class EmailTemplate(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(255), nullable=False)
    subject = db.Column(db.String(255), nullable=False)
    body = db.Column(db.Text, nullable=False)
    

@app.route('/send_template_email/<int:client_id>/<int:template_id>')
def send_template_email(client_id, template_id):
    client = Client.query.get_or_404(client_id)
    template = EmailTemplate.query.get_or_404(template_id)
    
    mail = Mail(app)
    msg = Message(template.subject,
                  sender='admin@versanstr.store',
                  recipients=[client.email])
    msg.body = "This is the plain text body of the email for clients that do not support HTML."
    msg.html = template.body  # Set the HTML content of the email
    mail.send(msg)
    
    flash('Email sent successfully to {}'.format(client.email), 'success')
    return redirect(url_for('expired_clients'))


@app.route('/select_template_for_client/<int:client_id>')
def select_template_for_client(client_id):
    templates = EmailTemplate.query.all()  # Fetch all templates
    return render_template('select_template.html', templates=templates, client_id=client_id)

@app.route('/send_email/<int:client_id>')
def send_email(client_id):
    # Redirect to the template selection page, passing along the client_id
    return redirect(url_for('select_template_for_client', client_id=client_id))




@app.route('/create_template', methods=['GET', 'POST'])
def create_template():
    if request.method == 'POST':
        # Here, process the form submission and save the new template
        name = request.form['name']
        subject = request.form['subject']
        body = request.form['body']
        new_template = EmailTemplate(name=name, subject=subject, body=body)
        db.session.add(new_template)
        db.session.commit()
        return redirect(url_for('template_list'))  # Redirect to the list of templates, for example
    
    return render_template('create_template.html')

@app.route('/template_list')
def template_list():
    templates = EmailTemplate.query.all()  # Assuming EmailTemplate is your model name
    return render_template('template_list.html', templates=templates)

@app.route('/view_template/<int:id>')
def view_template(id):
    template = EmailTemplate.query.get_or_404(id)  # Assuming EmailTemplate is your model
    return render_template('view_template.html', template=template)

@app.route('/edit_template/<int:id>', methods=['GET', 'POST'])
def edit_template(id):
    template = EmailTemplate.query.get_or_404(id)
    if request.method == 'POST':
        # Update the template with the new details from the form
        template.name = request.form['name']
        template.subject = request.form['subject']
        template.body = request.form['body']
        db.session.commit()
        flash('Template updated successfully.', 'success')
        return redirect(url_for('template_list'))  # Assuming you have a route named template_list to list templates
    # For a GET request, display the template edit form
    return render_template('edit_template.html', template=template)

@app.route('/delete_template/<int:id>', methods=['POST'])
def delete_template(id):
    template = EmailTemplate.query.get_or_404(id)
    db.session.delete(template)
    db.session.commit()
    flash('Template deleted successfully.', 'success')
    return redirect(url_for('template_list'))  # Redirect back to the list of templates


@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        username = request.form.get('username')
        password = request.form.get('password')
        if username == USER_CREDENTIALS['username'] and password == USER_CREDENTIALS['password']:
            session['username'] = username
            return redirect(url_for('index'))
        else:
            flash('Invalid credentials, please try again.', 'error')
    return render_template('login.html')

@app.route('/logout')
def logout():
    session.pop('username', None)
    return redirect(url_for('login'))

# Routes
@app.route('/')
def index():
    if 'username' not in session:
        return redirect(url_for('login'))
    clients = Client.query.all()
    alert_clients = [client for client in clients if client.expiry_date <= datetime.now().date() + timedelta(days=2)]
    return render_template('index.html', clients=clients, alert_clients=alert_clients)

@app.route('/add', methods=['GET', 'POST'])
def add_client():
    if 'username' not in session:
        return redirect(url_for('login'))

    panels = PanelName.query.all()

    if request.method == 'POST':
        if 'clientCSV' in request.form and request.form['clientCSV'].strip():
            csv_text = request.form['clientCSV']
            csv_file = csv.DictReader(StringIO(csv_text))
            
            skipped_clients = []
            added_clients = 0
            for row in csv_file:
                try:
                    new_client = Client(
                        client_number=row['client_number'],
                        phone=row['phone'],
                        email=row['email'],
                        iptv_username=row['iptv_username'],
                        panel_name=row['panel_name'],
                        expiry_date=datetime.strptime(row['expiry_date'], '%Y-%m-%d').date()
                    )
                    db.session.add(new_client)
                    db.session.commit()
                    added_clients += 1
                except IntegrityError:
                    db.session.rollback()  # Rollback the transaction
                    skipped_clients.append(row['client_number'])
                    continue  # Skip this iteration and continue with the next row

            if skipped_clients:
                flash(f'Some clients were not added due to duplicates: {", ".join(skipped_clients)}', 'warning')
            if added_clients:
                flash(f'{added_clients} clients added successfully!', 'success')

            return redirect(url_for('index'))
        else:
            # Existing logic for adding a single client through form fields
            new_client = Client(
                client_number=request.form['client_number'],
                phone=request.form['phone'],
                email=request.form['email'],
                iptv_username=request.form['iptv_username'],
                panel_name=request.form['panel_name'],
                expiry_date=datetime.strptime(request.form['expiry_date'], '%Y-%m-%d').date()
            )
            try:
                db.session.add(new_client)
                db.session.commit()
                flash('Client added successfully!', 'success')
            except IntegrityError:
                db.session.rollback()
                flash('This client number already exists.', 'error')

            return redirect(url_for('index'))
    
    return render_template('add_client.html', panels=panels)


@app.route('/search')
def search():
    if 'username' not in session:
        return redirect(url_for('login'))
    query = request.args.get('query', '')
    clients = Client.query.filter(
        or_(
            Client.client_number.contains(query),
            Client.phone.contains(query),
            Client.email.contains(query),
            Client.iptv_username.contains(query),
            Client.panel_name.contains(query)
        )
    ).all() if query else Client.query.all()
    return render_template('index.html', clients=clients, alert_clients=[])

@app.route('/edit/<int:id>', methods=['GET', 'POST'])
def edit_client(id):
    if 'username' not in session:
        return redirect(url_for('login'))
    client = Client.query.get_or_404(id)
    panels = PanelName.query.all()
    if request.method == 'POST':
        client.client_number = request.form['client_number']
        client.phone = request.form['phone']
        client.email = request.form['email']
        client.iptv_username = request.form['iptv_username']
        client.panel_name = request.form['panel_name']
        client.expiry_date = datetime.strptime(request.form['expiry_date'], '%Y-%m-%d').date()
        db.session.commit()
        flash('Client details updated successfully!', 'success')
        return redirect(url_for('index'))
    return render_template('edit_client.html', client=client, panels=panels)

@app.route('/delete/<int:id>')
def delete_client(id):
    if 'username' not in session:
        return redirect(url_for('login'))
    client = Client.query.get_or_404(id)
    db.session.delete(client)
    db.session.commit()
    flash('Client deleted successfully!', 'success')
    return redirect(url_for('index'))

@app.route('/add_panel_name', methods=['GET', 'POST'])
def add_panel_name():
    if 'username' not in session:
        return redirect(url_for('login'))
    if request.method == 'POST':
        panel_name = request.form.get('name')
        if panel_name:  # Check if the input is not empty
            new_panel = PanelName(name=panel_name)
            db.session.add(new_panel)
            db.session.commit()
            flash('New IPTV Panel Name added successfully!', 'success')
        else:
            flash('Panel Name cannot be empty!', 'error')
    panels = PanelName.query.all()
    return render_template('add_panel_name.html', panels=panels)

@app.route('/delete_panel_name/<int:panel_id>', methods=['POST'])
def delete_panel_name(panel_id):
    if 'username' not in session:
        return redirect(url_for('login'))
    panel_to_delete = PanelName.query.get_or_404(panel_id)
    db.session.delete(panel_to_delete)
    db.session.commit()
    flash('Panel Name deleted successfully!', 'success')
    return redirect(url_for('add_panel_name'))

def send_email(to, subject, body):
    try:
        msg = Message(subject, recipients=[to], body=body)
        mail.send(msg)
        log = EmailLog(recipient=to, subject=subject, status='Sent')
        db.session.add(log)
        db.session.commit()
    except Exception as e:
        log = EmailLog(recipient=to, subject=subject, status=f'Failed: {str(e)}')
        db.session.add(log)
        db.session.commit()


def check_and_notify_expired_clients():
    today = datetime.now().date()
    thirty_days_later = today + timedelta(days=30)
    
    # Fetch clients whose subscriptions have expired or are expiring within the next 30 days
    clients = Client.query.filter(Client.expiry_date <= thirty_days_later).all()
    
    for client in clients:
        if client.expiry_date < today:
            status = "expired"
        else:
            status = "expiring soon"
        
        subject = f"Subscription Expiration Alert for {client.client_number}"
        body = f"The subscription for {client.client_number}, {client.email}, has {status} on {client.expiry_date}."
        
        send_email('admin@versanpay.com', subject, body)

@scheduler.task('cron', id='daily_check', hour=0, minute=0)
def scheduled_check():
    with app.app_context():
        check_and_notify_expired_clients()


@app.route('/expired_clients')
def expired_clients():
    if 'username' not in session:
        return redirect(url_for('login'))

    today = date.today()
    # Assuming 'expiry_date' is a date field in your 'Client' model
    # Modify the filter to include clients whose expiry dates are before today (already expired)
    clients = Client.query.filter(Client.expiry_date <= (today + timedelta(days=30))).all()

    return render_template('expired_clients.html', clients=clients, today=today)   

@app.route('/email_logs')
def email_logs():
    if 'username' not in session:
        return redirect(url_for('login'))
    logs = EmailLog.query.order_by(EmailLog.timestamp.desc()).all()
    return render_template('email_logs.html', logs=logs)




if __name__ == "__main__":
    with app.app_context():
        db.create_all()
    scheduler.start()
    app.run(debug=True)
    app.debug = True

